﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using PlclikeTK;

namespace PlclikeTKTestCSherp
{
    /// <summary>
    /// PlclikeTK サンプルプログラム
    /// </summary>
    public partial class Form1 : Form
    {
        /// <summary>
        /// PlclikeTK 起動フォーム
        /// </summary>
        frmPLC fPlc;
        
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public Form1()
        {
            InitializeComponent();
        }

        #region "フォームイベント処理"

        /// <summary>
        /// FormClosingイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            // PlclikeTK 起動フォームが生成されている場合
            if (fPlc != null)
            {
                // 一応 PlclikeTK 起動フォームを終了しておきます。
                fPlc.Close();
                fPlc.Dispose();
            }
        }

        /// <summary>
        /// FormLoadイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_Load(object sender, EventArgs e)
        {

        }

        /// <summary>
        /// FormShownイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_Shown(object sender, EventArgs e)
        {
            // PlclikeTK 起動状態の表示
            DispRunStatus(0);
        }

        #endregion

        #region "コントロールイベント処理"

        /// <summary>
        /// Plclike起動処理ボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void StartButton_Click(object sender, EventArgs e)
        {
            // Plclike 起動処理
            if (StartPlclikeTK() == false)
            {
                return;
            }

            // Plclike起動状態の表示
            DispRunStatus(0);
        }

        /// <summary>
        /// Plclike実行・停止ボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void runStopButton_Click(object sender, EventArgs e)
        {
            int iRes = -1;
            string message = "";

            // 仮想PLC情報の有無
            if (fPlc == null)
            {
                // 仮想PLC起動確認警告
                message = "仮想PLC情報が実行されていません。まず、「実行」ボタンをクリックし仮想PLCを起動してください。";
                AddLoggingMessage(message);
                return;
            }
            else if (fPlc.IsDisposed == true)
            {
                // 仮想PLC起動確認警告
                message = "仮想PLC情報が実行されていません。まず、「実行」ボタンをクリックし仮想PLCを起動してください。";
                AddLoggingMessage(message);
                return;
            }

            // 起動／実行確認
            if (fPlc.RunMode == 1)
            {
                // 通信切断開始
                iRes = fPlc.StopRun();
                if (iRes == 0)
                {
                    message = "仮想PLCを停止しました。";
                    AddLoggingMessage(message);
                }
                else
                {
                    message = "仮想PLCを停止時にエラーが発生しました。" + fPlc.ErrorMessage;
                    AddLoggingMessage(message);
                }
            }
            else
            {
                // 値設定情報が作成されていないようなら設定する
                if ( this.comboDevice.Items.Count<=0 )
                {
                    // 設定情報取得
                    SetCommboBoxDevice(true);
                    this.textAddress.Text = "0000";
                    this.textValue.Text = "0000";
                }
                // 通信接続開始
                iRes = fPlc.StartRun();
                if (iRes == 0)
                {
                    message = "仮想PLCを実行しました。";
                    AddLoggingMessage(message);
                }
                else
                {
                    message = "仮想PLCを実行時にエラーが発生しました。" + fPlc.ErrorMessage;
                    AddLoggingMessage(message);
                }
            }

            DispRunStatus(0);
        }

        /// <summary>
        /// メモリ読出しボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MemoryReadButton_Click(object sender, EventArgs e)
        {
            string sDevice = this.comboDevice.Text;
            string sAddress = this.textAddress.Text;
            string sValue = "";
            string message = "";
            int iCh = 1;

            // 仮想PLC情報の有無
            if (fPlc == null)
            {
                // 仮想PLC起動確認警告
                message = "仮想PLC情報が実行されていません。まず、「実行」ボタンをクリックし仮想PLCを起動してください。";
                AddLoggingMessage(message);
                return;
            }
            else if (fPlc.IsDisposed == true)
            {
                // 仮想PLC起動確認警告
                message = "仮想PLC情報が実行されていません。まず、「実行」ボタンをクリックし仮想PLCを起動してください。";
                AddLoggingMessage(message);
                return;
            }

            // アドレス情報取得
            sValue = fPlc.GetMemoryAddress(sDevice + sAddress, iCh);
            this.textValue.Text = sValue;
        }

        /// <summary>
        /// メモリ書込みボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MemoryWriteButton_Click(object sender, EventArgs e)
        {
            string sDevice = this.comboDevice.Text;
            string sAddress = this.textAddress.Text;
            string sValue = this.textValue.Text;
            string message = "";
            int iCh = 1;
            int iRes = -1;

            // 仮想PLC情報の有無
            if (fPlc == null)
            {
                // 仮想PLC起動確認警告
                message = "仮想PLC情報が実行されていません。まず、「実行」ボタンをクリックし仮想PLCを起動してください。";
                AddLoggingMessage(message);
                return;
            }
            else if (fPlc.IsDisposed == true)
            {
                // 仮想PLC起動確認警告
                message = "仮想PLC情報が実行されていません。まず、「実行」ボタンをクリックし仮想PLCを起動してください。";
                AddLoggingMessage(message);
                return;
            }

            // アドレス情報取得
            iRes = fPlc.PutMemoryAddress(sDevice + sAddress, iCh, sValue);
            if( iRes == 0)
            {
                message = "メモリ設定完了";
            }
            else
            {
                message = "メモリ設定失敗：" + fPlc.ErrorMessage;
            }
            AddLoggingMessage(message);
        }

        /// <summary>
        /// 処理内容のクリアボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MessageClearButton_Click(object sender, EventArgs e)
        {
            // 処理内容クリア処理
            ClrLoggingMessage();
        }

        #endregion

        #region "内部処理"

        /// <summary>
        /// PlclikeTK 実行状態取得
        /// </summary>
        /// <returns>0=停止、1=実行、それ以外=未定義</returns>
        private int GetRunStatus()
        {
            int iRes = -1;
            string sMessage = "";
            try
            {
                if (fPlc == null)
                {
                    // 未定義
                    iRes = -1;
                }
                else if (fPlc.IsDisposed == true)
                {
                    // 未定義
                    iRes = -1;
                }
                else if (fPlc.RunMode == 1)
                {
                    // 起動中
                    iRes = 1;
                }
                else
                {
                    // 停止中
                    iRes = 0;
                }
            }
            catch(Exception e)
            {
                // 例外エラー
                iRes = -2;
                sMessage = e.Message;
            }
            return iRes;
        }

        /// <summary>
        /// PlclikeTK 実行状態取得
        /// </summary>
        /// <returns>状態情報</returns>
        private string GetRunStatusName()
        {
            string sMessage = "";
            try
            {
                if (fPlc == null)
                {
                    sMessage = "未処理";
                }
                else if (fPlc.IsDisposed == true)
                {
                    sMessage = "未処理";
                }
                else if (fPlc.RunMode == 1)
                {
                    sMessage = "起動中";
                }
                else
                {
                    sMessage = "停止中";
                }
            }
            catch(Exception e)
            {
                sMessage = "例外エラー：" + e.Message;
            }
            return sMessage;
        }

        /// <summary>
        /// PlclikeTK 起動状態
        /// </summary>
        /// <param name="iFlag">0=Open/Closes処理時,1=Dispose処理時</param>
        private void DispRunStatus(int iFlag)
        {
            // 接続状態表示
            string sMessage = GetRunStatusName();
            this.lblRunStatus.Text = "PlclikeTKの状態：" + sMessage;
            // 起動・停止ボタン表示切替
            if (GetRunStatus() == 1)
            {
                this.runStopButton.Text = "接続停止";
                this.runStopButton.ForeColor = Color.Red;
            }
            else
            {
                this.runStopButton.Text = "実行開始";
                this.runStopButton.ForeColor = Color.Blue;
            }
        }

        /// <summary>
        /// PlclikeTK 初期処理
        /// </summary>
        private void InitPlclikeTK()
        {
            // PlclikeTK 起動フォームが未生成なので生成します。
            fPlc = new PlclikeTK.frmPLC();
            fPlc.dgtTransferOpenClose += ChangeTransferOpenClose;
            fPlc.dgtTransferMemory += ChangePLCData;
            fPlc.Disposed += ChangeTransferDispose;
        }

        /// <summary>
        /// PlclikeTK 起動
        /// </summary>
        private bool StartPlclikeTK()
        {
            bool bRes = false;
            string message = "";

            // 処理内容へ表示
            message = "PlclikeTK 起動開始";
            AddLoggingMessage(message);

            // PlclikeTK 起動フォーム生成判定
            if (fPlc == null)
            {
                // PlclikeTK 起動フォームが未生成なので生成します。
                InitPlclikeTK();
            }
            else if (fPlc.IsDisposed == true)
            {
                // PlclikeTK 起動フォームが未生成なので生成します。
                InitPlclikeTK();
            }
            else
            {
                message = "PlclikeTK はすでに実行中です";
                AddLoggingMessage(message);
                if(this.radioButton1.Checked == true)
                {
                    if( fPlc.Visible == false)
                    {
                        message = "PlclikeTK はすでに実行中ですが、PlclikeTK は非表示です。\r\n";
                        message += "PlilikeTM を表示しますか？";
                        if( MessageBox.Show(message,"起動確認", MessageBoxButtons.YesNo
                            , MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
                        {
                            fPlc.Visible = true;
                        }
                        return true;
                    }
                }
                else
                {
                    if (fPlc.Visible == true)
                    {
                        message = "PlclikeTK はすでに実行中です。\r\n";
                        message += "PlilikeTM を非表示しますか？";
                        if (MessageBox.Show(message, "起動確認", MessageBoxButtons.YesNo
                            , MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
                        {
                            fPlc.Visible = false;
                        }
                        return true;
                    }
                }
                return true;
            }

            // PlclikeTK 起動フォーム実行判定
            if (this.radioButton1.Checked == true)
            {
                // PlclikeTK 起動フォームを表示して実行
                message = "PlclikeTK 起動画面を表示して実行";
                AddLoggingMessage(message);
                fPlc.Show();
            }
            else
            {
                // PlclikeTK 起動フォームを非表示で実行
                message = "PlclikeTK 起動画面を非表示して実行";
                AddLoggingMessage(message);
                fPlc.ShowHide();
            }

            // PlclikeTK ライセンス確認
            if (fPlc.LicenseFlag == false)
            {
                if (MessageBox.Show("ライセンス認証されていないようです・・・\r\n" +
                    "ライセンス認証されていない場合、設定ファイル等は使用できませんので設定する必要があります。\r\n" +
                    "設定する場合は「はい」、中断する場合は「いいえ」を選択してください", "ライセンス確認",
                    MessageBoxButtons.YesNo, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button2) == DialogResult.No)
                {
                    // 処理の中断
                    return false;
                }

                // PlclikeTK 通信設定を実行
                bRes = fPlc.StartTransferConfig();
                if (bRes == false)
                {
                    MessageBox.Show("通信設定をキャンセル、または設定失敗だったので処理を中断します・・・",
                        "設定確認", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return false;
                }
            }

            // 設定情報取得
            SetCommboBoxDevice(true);
            this.textAddress.Text = "0000";
            this.textValue.Text = "0000";
            return true;
        }

        /// <summary>
        /// デバイス名称生成
        /// </summary>
        /// <param name="default_value">既定値使用区分(true=設定,false=未設定)</param>
        private void SetCommboBoxDevice(bool default_value)
        {
            int iMax = -1;
            int iLp = -1;
            string message = "";
            string value = "";

            // 仮想PLC情報の有無
            if (fPlc == null)
            {
                // 仮想PLC起動確認警告
                message = "仮想PLC情報が実行されていません。まず、「実行」ボタンをクリックし仮想PLCを起動してください。";
                AddLoggingMessage(message);
                return;
            }
            else if (fPlc.IsDisposed == true)
            {
                // 仮想PLC起動確認警告
                message = "仮想PLC情報が実行されていません。まず、「実行」ボタンをクリックし仮想PLCを起動してください。";
                AddLoggingMessage(message);
                return;
            }

            // 値設定初期化
            iMax = fPlc.GetDeviceCount();
            this.comboDevice.Items.Clear();
            for (iLp = 0; iLp < iMax; iLp++)
            {
                value = fPlc.GetDeviceName(iLp);
                this.comboDevice.Items.Add(value);
            }
            if(default_value==true)
            {
                if ( this.comboDevice.Items.Count>=1)
                {
                    this.comboDevice.SelectedIndex = 0;
                }
            }
        }

        /// <summary>
        /// 処理内容メッセージ初期化
        /// </summary>
        private void ClrLoggingMessage()
        {
            this.textPlclike.Text = "";
        }

        /// <summary>
        /// 処理内容メッセージ追記
        /// </summary>
        /// <param name="value">メッセージ文字列</param>
        private void AddLoggingMessage(string value)
        {
            // 既定は、先頭へメッセージ文字列を追記します。は
            AddLoggingessage(value, 0, true);
        }

        /// <summary>
        /// 処理内容メッセージ追記
        /// </summary>
        /// <param name="value">メッセージ文字列</param>
        /// <param name="top_or_bottom">メッセージ追記区分(0=先頭に追記,1=終端へ追記)</param>
        /// <param name="disp_time">処理内容に日時付加有無(true=日時を追加,false=日時を非表示)</param>
        private void AddLoggingessage(string value, int top_or_bottom, bool disp_time)
        {
            // メッセージ指定確認
            if (String.IsNullOrEmpty(value) == true)
            {
                // メッセージが無い場合は処理を中断
                return;
            }
            string message = "";
            // 日時表記有無
            if( disp_time==true)
            {
                message = DateTime.Now.ToString("yyyy/MM/dd HH:mm:ss") + " ";
            }
            message += value;
            // メッセージ追加
            if (top_or_bottom == 1)
            {
                this.textPlclike.Text += "\r\n" + message;
            }
            else
            {
                this.textPlclike.Text = message + "\r\n" + this.textPlclike.Text;
            }
        }

        /// <summary>
        /// PlclikeTK 起動/停止 切替イベント
        /// </summary>
        /// <param name="iSerialNo">登録番号</param>
        private void ChangeTransferOpenClose(int iSerialNo)
        {
            // PlclikeTK 起動状態確認
            DispRunStatus(0);
        }

        /// <summary>
        /// PlclikeTK 終了時 イベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ChangeTransferDispose(object sender, EventArgs e)
        {
            // PlclikeTK 起動状態確認
            DispRunStatus(1);
        }

        /// <summary>
        /// 書込みイベント処理
        /// </summary>
        /// <param name="iSerialNo">シリアル番号</param>
        /// <param name="sDevice">デバイス</param>
        /// <param name="sAddress">アドレス</param>
        /// <param name="sMemoryData">書込値</param>
        private void ChangePLCData(int iSerialNo, string sDevice, string sAddress, string sMemoryData)
        {
            if (this.InvokeRequired == true)
            {
                frmPLC.SetTransferMemory d = new frmPLC.SetTransferMemory(ChangePLCDataSub);
                this.BeginInvoke(d, new object[] { iSerialNo, sDevice, sAddress, sMemoryData });
            }
            else
            {
                ChangePLCDataSub(iSerialNo, sDevice, sAddress, sMemoryData);
            }
        }

        /// <summary>
        /// 書込みイベント処理
        /// </summary>
        /// <param name="iSerialNo">シリアル番号</param>
        /// <param name="sDevice">デバイス</param>
        /// <param name="sAddress">アドレス</param>
        /// <param name="sMemoryData">書込値</param>
        public void ChangePLCDataSub(int iSerialNo, string sDevice, string sAddress, string sMemoryData)
        {
            string message = "";

            // イベント取得情報を代入
            message = string.Format(" PLC書込みイベント: SerialNo[{0}] Device[{1}] Address[{2}] DATA[{3}]", iSerialNo.ToString(), sDevice, sAddress, sMemoryData);
            // 処理内容へ表示
            AddLoggingMessage(message);
        }


        #endregion

    }
}
