﻿''' <summary>
''' Plclike サンプルプログラム
''' </summary>
Public Class Form1

    ''' <summary>
    ''' Plclike 起動フォーム
    ''' </summary>
    Private fPlc As PlclikeTK.frmPLC

#Region "フォームイベント処理"

    ''' <summary>
    ''' FormClosingイベント
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub Form1_FormClosing(sender As Object, e As FormClosingEventArgs) Handles MyBase.FormClosing
        '' Plclike 起動フォームが生成されている場合
        If (Not fPlc Is Nothing) Then
            '' 一応 Plclike 起動フォームを終了しておきます。
            fPlc.Close()
            fPlc.Dispose()
        End If
    End Sub

    ''' <summary>
    '''  FormLoadイベント
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load

    End Sub

    ''' <summary>
    ''' FormShownイベント
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub Form1_Shown(sender As Object, e As EventArgs) Handles MyBase.Shown
        '' Plclike起動状態の表示
        DispRunStatus(0)
    End Sub


#End Region

#Region "コントロールイベント処理"

    ''' <summary>
    ''' Plclike起動処理ボタンイベント
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub StartButton_Click(sender As Object, e As EventArgs) Handles StartButton.Click
        '' Plclike 起動処理
        If (StartPlclike() = False) Then
            Return
        End If

        '' Plclike起動状態の表示
        DispRunStatus(0)
    End Sub

    ''' <summary>
    ''' Plclike実行・停止ボタンイベント
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub runStopButton_Click(sender As Object, e As EventArgs) Handles runStopButton.Click
        Dim iRes As Integer = -1
        Dim message As String = ""

        '' 仮想PLC情報の有無
        If (fPlc Is Nothing) Then
            '' 仮想PLC起動確認警告
            message = "仮想PLC情報が実行されていません。まず、「実行」ボタンをクリックし仮想PLCを起動してください。"
            AddLoggingMessage(message)
            Return
        ElseIf (fPlc.IsDisposed = True) Then

            '' 仮想PLC起動確認警告
            message = "仮想PLC情報が実行されていません。まず、「実行」ボタンをクリックし仮想PLCを起動してください。"
            AddLoggingMessage(message)
            Return
        End If

        '' 実行／停止確認
        If (fPlc.RunMode = 1) Then
            '' 通信切断開始
            iRes = fPlc.StopRun()
            If (iRes = 0) Then
                message = "仮想PLCを停止しました。"
                AddLoggingMessage(message)
            Else
                message = "仮想PLCを停止時にエラーが発生しました。" + fPlc.ErrorMessage
                AddLoggingMessage(message)
            End If
        Else
            '' 値設定情報が作成されていないようなら設定する
            If (Me.comboDevice.Items.Count <= 0) Then
                '' 設定情報取得
                SetCommboBoxDevice(True)
                Me.textAddress.Text = "0000"
                Me.textValue.Text = "0000"
            End If
            '' 通信接続開始
            iRes = fPlc.StartRun()
            If (iRes = 0) Then
                message = "仮想PLCを実行しました。"
                AddLoggingMessage(message)
            Else
                message = "仮想PLCを実行時にエラーが発生しました。" + fPlc.ErrorMessage
                AddLoggingMessage(message)
            End If
            DispRunStatus(0)
        End If
    End Sub

    ''' <summary>
    ''' メモリ読出しボタンイベント
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub MemoryReadButton_Click(sender As Object, e As EventArgs) Handles MemoryReadButton.Click
        Dim sDevice As String = Me.comboDevice.Text
        Dim sAddress As String = Me.textAddress.Text
        Dim sValue As String = ""
        Dim message As String = ""
        Dim iCh As Integer = 1

        '' 仮想PLC情報の有無
        If (fPlc Is Nothing) Then
            '' 仮想PLC起動確認警告
            message = "仮想PLC情報が実行されていません。まず、「実行」ボタンをクリックし仮想PLCを起動してください。"
            AddLoggingMessage(message)
            Return
        ElseIf (fPlc.IsDisposed = True) Then
            '' 仮想PLC起動確認警告
            message = "仮想PLC情報が実行されていません。まず、「実行」ボタンをクリックし仮想PLCを起動してください。"
            AddLoggingMessage(message)
            Return
        End If
        '' アドレス情報取得
        sValue = fPlc.GetMemoryAddress(sDevice + sAddress, iCh)
        Me.textValue.Text = sValue
    End Sub

    ''' <summary>
    ''' メモリ書込みボタンイベント
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub MemoryWriteButton_Click(sender As Object, e As EventArgs) Handles MemoryWriteButton.Click
        Dim sDevice As String = Me.comboDevice.Text
        Dim sAddress As String = Me.textAddress.Text
        Dim sValue As String = Me.textValue.Text
        Dim message As String = ""
        Dim iCh As Integer = 1
        Dim iRes As Integer = -1

        '' 仮想PLC情報の有無
        If (fPlc Is Nothing) Then
            '' 仮想PLC起動確認警告
            message = "仮想PLC情報が実行されていません。まず、「実行」ボタンをクリックし仮想PLCを起動してください。"
            AddLoggingMessage(message)
            Return

        ElseIf (fPlc.IsDisposed = True) Then

            '' 仮想PLC起動確認警告
            message = "仮想PLC情報が実行されていません。まず、「実行」ボタンをクリックし仮想PLCを起動してください。"
            AddLoggingMessage(Message)
            Return
        End If

        '' アドレス情報取得
        iRes = fPlc.PutMemoryAddress(sDevice + sAddress, iCh, sValue)
        If (iRes = 0) Then
            message = "メモリ設定完了"
        Else
            message = "メモリ設定失敗：" + fPlc.ErrorMessage
            AddLoggingMessage(message)
        End If
    End Sub

    ''' <summary>
    ''' 処理内容のクリアボタンイベント
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub MessageClearButton_Click(sender As Object, e As EventArgs) Handles MessageClearButton.Click
        '' 処理内容クリア処理
        ClrLoggingMessage()
    End Sub

#End Region

#Region "内部処理"

    ''' <summary>
    ''' Plclike 実行状態取得
    ''' </summary>
    ''' <returns>0=停止、1=実行、それ以外=未定義</returns>
    Private Function GetRunStatus() As Integer
        Dim iRes As Integer = -1
        Dim sMessage As String = ""
        Try
            If (fPlc Is Nothing) Then
                iRes = -1
            ElseIf (fPlc.RunMode = 1) Then
                '' 起動中
                iRes = 1
            Else
                '' 停止中
                iRes = 0
            End If
        Catch e As Exception
            iRes = -2
            sMessage = e.Message
        End Try

        Return iRes
    End Function

    ''' <summary>
    ''' Plclike 実行状態取得
    ''' </summary>
    ''' <returns>状態情報</returns>
    Private Function GetRunStatusName() As String
        Dim iRes As Integer = -1
        Dim sMessage As String = ""
        Try
            If (fPlc Is Nothing) Then
                iRes = -1
                sMessage = "未処理"
            ElseIf (fPlc.RunMode = 1) Then
                '' 起動中
                iRes = 1
                sMessage = "起動中"
            Else
                '' 停止中
                iRes = 0
                sMessage = "停止中"
            End If
        Catch e As Exception
            iRes = -2
            sMessage = e.Message
        End Try

        Return sMessage
    End Function

    ''' <summary>
    ''' Plclike 起動状態
    ''' </summary>
    ''' <param name="iFlag">0=Open/Closes処理時,1=Dispose処理時</param>
    Private Sub DispRunStatus(iFlag As Integer)
        '' 接続状態表示
        Dim sMessage As String = GetRunStatusName()
        Me.lblRunStatus.Text = "Plclikeの状態：" + sMessage
        '' 起動・停止ボタン表示切替
        If (GetRunStatus() = 1) Then
            Me.runStopButton.Text = "接続停止"
            Me.runStopButton.ForeColor = Color.Red
        Else
            Me.runStopButton.Text = "実行開始"
            Me.runStopButton.ForeColor = Color.Blue
        End If
    End Sub

    ''' <summary>
    ''' Plclike 初期処理
    ''' </summary>
    Private Sub InitPlclike()
        '' Plclike 起動フォームが未生成なので生成します。
        fPlc = New PlclikeTK.frmPLC()
        fPlc.dgtTransferOpenClose = New PlclikeTK.frmPLC.SetTransferOpenClose(AddressOf ChangeTransferOpenClose)
        fPlc.dgtTransferMemory = New PlclikeTK.frmPLC.SetTransferMemory(AddressOf ChangePLCData)
        'fPlc.Disposed = New EventHandler(AddressOf ChangeTransferDispose)
    End Sub

    ''' <summary>
    ''' Plclike 起動
    ''' </summary>
    ''' <returns></returns>
    Private Function StartPlclike() As Boolean
        Dim bRes As Boolean = False
        Dim message As String = ""

        ''処理内容へ表示
        message = "Plclike 起動開始"
        AddLoggingMessage(message)

        '' Plclike 起動フォーム生成判定
        If (fPlc Is Nothing) Then
            '' Plclike 起動フォームが未生成なので生成します。
            InitPlclike()
        ElseIf (fPlc.IsDisposed = True) Then
            '' Plclike 起動フォームが未生成なので生成します。
            InitPlclike()

        Else
            '' Plclike 実行中
            message = "Plclike はすでに実行中です"
            AddLoggingMessage(message)
            If (Me.radioButton1.Checked = True) Then
                If (fPlc.Visible = False) Then
                    message = "Plclike はすでに実行中ですが、Plclike は非表示です。" + vbCrLf
                    message += "Plilike を表示しますか？"
                    If (MessageBox.Show(message, "起動確認", MessageBoxButtons.YesNo _
                            , MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button2) = DialogResult.Yes) Then
                        fPlc.Visible = True
                    End If
                    Return True
                End If
            Else
                If (fPlc.Visible = True) Then
                    message = "Plclike はすでに実行中です。" + vbCrLf
                    message += "Plilike を非表示しますか？"
                    If (MessageBox.Show(message, "起動確認", MessageBoxButtons.YesNo _
                        , MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button2) = DialogResult.Yes) Then
                        fPlc.Visible = False
                    End If
                    Return True
                End If
            End If
            Return True
        End If

        '' Plclike 起動フォーム実行判定
        If (Me.radioButton1.Checked = True) Then
            '' Plclike 起動フォームを表示して実行
            message = "Plclike 起動画面を表示して実行"
            AddLoggingMessage(message)
            fPlc.Show()
        Else
            '' PlcLike 起動フォームを非表示で実行
            message = "Plclike 起動画面を非表示して実行"
            AddLoggingMessage(message)
            fPlc.ShowHide()
        End If

        '' Plclike ライセンス確認
        If (fPlc.LicenseFlag = False) Then
            If (MessageBox.Show("ライセンス認証されていないようです・・・" + vbCrLf _
                    + "ライセンス認証されていない場合、設定ファイル等は使用できませんので設定する必要があります。" + vbCrLf _
                    + "設定する場合は「はい」、中断する場合は「いいえ」を選択してください", "ライセンス確認",
                    MessageBoxButtons.YesNo, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button2) = DialogResult.No) Then
                '' 処理の中断
                Return False
            End If

            '' Plclike 通信設定を実行
            bRes = fPlc.StartTransferConfig()
            If (bRes = False) Then
                MessageBox.Show("通信設定をキャンセル、または設定失敗だったので処理を中断します・・・",
                        "設定確認", MessageBoxButtons.OK, MessageBoxIcon.Information)
                Return False
            End If
        End If

        '' 設定情報取得
        SetCommboBoxDevice(True)
        Me.textAddress.Text = "0000"
        Me.textValue.Text = "0000"
        Return True
    End Function

    ''' <summary>
    ''' デバイス名称生成
    ''' </summary>
    ''' <param name="default_value">既定値使用区分(true=設定,false=未設定)</param>
    Private Sub SetCommboBoxDevice(default_value As Boolean)
        Dim iMax As Integer = -1
        Dim iLp As Integer = -1
        Dim message As String = ""
        Dim value As String = ""

        '' 仮想PLC情報の有無
        If (fPlc Is Nothing) Then
            '' 仮想PLC起動確認警告
            message = "仮想PLC情報が実行されていません。まず、「実行」ボタンをクリックし仮想PLCを起動してください。"
            AddLoggingMessage(message)
            Return
        ElseIf (fPlc.IsDisposed = True) Then
            '' 仮想PLC起動確認警告
            message = "仮想PLC情報が実行されていません。まず、「実行」ボタンをクリックし仮想PLCを起動してください。"
            AddLoggingMessage(message)
            Return
        End If

        '' 値設定初期化
        iMax = fPlc.GetDeviceCount()
        Me.comboDevice.Items.Clear()
        For iLp = 0 To (iMax - 1)
            value = fPlc.GetDeviceName(iLp)
            Me.comboDevice.Items.Add(value)
        Next iLp

        If (default_value = True) Then
            If (Me.comboDevice.Items.Count >= 1) Then
                Me.comboDevice.SelectedIndex = 0
            End If
        End If
    End Sub

    ''' <summary>
    ''' 処理内容メッセージ初期化
    ''' </summary>
    Private Sub ClrLoggingMessage()
        Me.textPlclike.Text = ""
    End Sub

    ''' <summary>
    ''' 処理内容メッセージ追記
    ''' </summary>
    ''' <param name="value">メッセージ文字列</param>
    Private Sub AddLoggingMessage(value As String)
        '' 既定は、先頭へメッセージ文字列を追記します。
        AddLoggingessage(value, 0, True)
    End Sub

    ''' <summary>
    ''' 処理内容メッセージ追記
    ''' </summary>
    ''' <param name="value">メッセージ文字列</param>
    ''' <param name="top_or_bottom">メッセージ追記区分(0=先頭に追記,1=終端へ追記)</param>
    ''' <param name="disp_time">処理内容に日時付加有無(true=日時を追加,false=日時を非表示)</param>
    Private Sub AddLoggingessage(value As String, top_or_bottom As Integer, disp_time As Boolean)

        '' メッセージ指定確認
        If (String.IsNullOrEmpty(value) = True) Then
            '' メッセージが無い場合は処理を中断
            Return
        End If
        Dim message As String = ""

        '' 日時表記有無
        If (disp_time = True) Then
            message = DateTime.Now.ToString("yyyy/MM/dd HH:mm:ss") + " "
        End If
        message += value
        '' メッセージ追加
        If (top_or_bottom = 1) Then
            Me.textPlclike.Text += vbCrLf + message
        Else
            Me.textPlclike.Text = message + vbCrLf + Me.textPlclike.Text
        End If
    End Sub

    ''' <summary>
    ''' Plclike 起動/停止 切替イベント
    ''' </summary>
    ''' <param name="iSerialNo">登録番号</param>
    Private Sub ChangeTransferOpenClose(iSerialNo As Integer)
        '' Plclike 起動状態確認
        DispRunStatus(0)
    End Sub

    ''' <summary>
    ''' Plclike 終了時 イベント
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub ChangeTransferDispose(sender As Object, e As EventArgs)
        '' Plclike 起動状態確認
        DispRunStatus(1)
    End Sub

    ''' <summary>
    ''' 書込みイベント処理
    ''' </summary>
    ''' <param name="iSerialNo">シリアル番号</param>
    ''' <param name="sDevice">デバイス</param>
    ''' <param name="sAddress">アドレス</param>
    ''' <param name="sMemoryData">書込値</param>
    Private Sub ChangePLCData(iSerialNo As Integer, sDevice As String, sAddress As String, sMemoryData As String)
        If (Me.InvokeRequired = True) Then
            Dim d As New PlclikeTK.frmPLC.SetTransferMemory(AddressOf ChangePLCDataSub)
            Dim ar As IAsyncResult = Me.BeginInvoke(d, New Object() {iSerialNo, sDevice, sAddress, sMemoryData})
            'Dim d As PlclikeTK.frmPLC.SetTransferMemory = New PlclikeTK.frmPLC.SetTransferMemory(ChangePLCDataSub)
            'Me.BeginInvoke(d, New Object[] { iSerialNo, sDevice, sAddress, sMemoryData })
        Else
            ChangePLCDataSub(iSerialNo, sDevice, sAddress, sMemoryData)
        End If

    End Sub

    ''' <summary>
    ''' 書込みイベント処理
    ''' </summary>
    ''' <param name="iSerialNo">シリアル番号</param>
    ''' <param name="sDevice">デバイス</param>
    ''' <param name="sAddress">アドレス</param>
    ''' <param name="sMemoryData">書込値</param>
    Public Sub ChangePLCDataSub(iSerialNo As Integer, sDevice As String, sAddress As String, sMemoryData As String)
        Dim message As String = ""
        '' イベント取得情報を代入
        message = String.Format(" PLC書込みイベント: SerialNo[{0}] Device[{1}] Address[{2}] DATA[{3}]", iSerialNo.ToString(), sDevice, sAddress, sMemoryData)
        '' 処理内容へ表示
        AddLoggingMessage(message)
    End Sub

#End Region

End Class
